from PlugIns import PluggableContainer
from SheetProviders import SheetProviderContainer
from Globals import HTMLFile, default__class_init__
from OFS.Folder import Folder

class Rack(SheetProviderContainer, PluggableContainer):

    """Base Rack - supports sheet provider plugins"""

    # Template (i.e. abstract) methods, should be left as-is

    def getItem(self, key):
        item = self.retrieveItem(key)
        if item is not None: item._setRack(self.aq_inner)
        return item

    def newItem(self, key=None):
        if key is None:
            key=self.newKey()
            if key is None: return None

        item = self.createItem(key)
        if item is not None: item._setRack(self.aq_inner)
        return item


    # Methods that must be overridden to implement behavior

    def retrieveItem(self,key):
        pass    # must be defined in subclass

    def createItem(self,key):
        """Create a new object, identified by key"""
        pass    # must be defined if ability to add is desired

    def newKey(self):
        """Generate a new object key"""
        pass    # must be defined if autogeneration desired



    __ac_permissions__ = (
        ('View management screens', ('manage_classForm','manage_class')),
        ('Access contents information', ('getItem','retrieveItem'), ),
        ('Add Items to Racks', ('newItem','createItem','newKey'), ),
    )


    # ZClass Machinery (Optionally overrideable)

    def _RawItem(self, key):
        """Create an empty object of the right class"""
        c = getattr(self, '_zclass', None)
        if c: return c._zclass_(key)
        else: return self._defaultClass(key)

    def manage_product_zclass_info(self):
        """Return a list of ZClasses suitable for use in this Rack"""
        r=[]
        Z=getattr(self, '_zclass', None)
        Z=getattr(Z, 'aq_self', Z)
        for d in self.aq_acquire('_getProductRegistryData')('zclasses'):
            z=d['meta_class']
            if not self._zclassOK(z):
                continue
            x={}
            x.update(d)
            x['selected'] = (z is Z) and 'selected' or ''
            del x['meta_class']
            r.append(x)
        return r

    def _zclassOK(self, z):
        """Verify that the ZClass selected is acceptable for this kind of rack"""
        return 1







    manage_classForm = HTMLFile('www/ItemClass', globals())

    def manage_class(self, zclass, REQUEST=None):
        """Change class of returned items"""

        self._zclass = None
        for d in self.aq_acquire('_getProductRegistryData')('zclasses'):
            if ("%s/%s" % (d.get('product'),d.get('id'))) == zclass:
                self._zclass=d['meta_class']
                break

        if REQUEST is not None:
            return self.manage_main(self, REQUEST, update_menu=1,
                message='Changed Class.')


    manage_options_right = (
                {'label':'Class',   'action':'manage_classForm'},
    ) + PluggableContainer.manage_options_right



default__class_init__(Rack)


















class RackMountable:
    """Mixin for classes that want to delegate
       storage issues to their Rack"""
       
    def _setRack(self, source):
        """Called by the rack before returning the object"""
        self._v_rack = source

    def PropertySheets(self):
        """Returns a 'propertysheets'-type object"""
        return self._v_rack.PropertySheetsFor(self)

    def PropertySheet(self, name='', xmlns=''):
        """Returns a specific property sheet"""
        return self._v_rack.PropertySheetFor(self, name, xmlns)








